/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2025                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2014                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2019-2021                                                **
**  RWTH Aachen University, IT Center                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SCOUT_PATTERNS_GEN_H
#define SCOUT_PATTERNS_GEN_H


namespace scout
{
// --- Constants ------------------------------------------------------------

const long PAT_NONE                                = -1;
const long PAT_TIME                                = 0;
const long PAT_VISITS                              = 1;
const long PAT_EXECUTION                           = 2;
const long PAT_OVERHEAD                            = 3;
const long PAT_COMP                                = 4;
const long PAT_ASYNC                               = 5;
const long PAT_MPI                                 = 6;
const long PAT_MPI_MANAGEMENT                      = 7;
const long PAT_MPI_INIT_EXIT                       = 8;
const long PAT_MPI_INIT_COMPLETION                 = 9;
const long PAT_MPI_FINALIZE_WAIT                   = 10;
const long PAT_MPI_MGMT_COMM                       = 11;
const long PAT_MPI_MGMT_FILE                       = 12;
const long PAT_MPI_MGMT_WIN                        = 13;
const long PAT_MPI_SYNCHRONIZATION                 = 14;
const long PAT_MPI_SYNC_COLLECTIVE                 = 15;
const long PAT_MPI_BARRIER_WAIT                    = 16;
const long PAT_MPI_BARRIER_COMPLETION              = 17;
const long PAT_MPI_COMMUNICATION                   = 18;
const long PAT_MPI_POINT2POINT                     = 19;
const long PAT_MPI_LATESENDER                      = 20;
const long PAT_MPI_LATESENDER_WO                   = 21;
const long PAT_MPI_LSWO_DIFFERENT                  = 22;
const long PAT_MPI_LSWO_SAME                       = 23;
const long PAT_MPI_LATERECEIVER                    = 24;
const long PAT_MPI_COLLECTIVE                      = 25;
const long PAT_MPI_EARLYREDUCE                     = 26;
const long PAT_MPI_EARLYSCAN                       = 27;
const long PAT_MPI_LATEBROADCAST                   = 28;
const long PAT_MPI_WAIT_NXN                        = 29;
const long PAT_MPI_NXN_COMPLETION                  = 30;
const long PAT_MPI_IO                              = 31;
const long PAT_MPI_IO_INDIVIDUAL                   = 32;
const long PAT_MPI_IO_COLLECTIVE                   = 33;
const long PAT_OMP_TARGET_KERNEL_EXECUTIONS        = 34;
const long PAT_OMP_IDLE_THREADS                    = 35;
const long PAT_OMP_LIMITED_PARALLELISM             = 36;
const long PAT_OMP_TIME                            = 37;
const long PAT_OMP_MANAGEMENT                      = 38;
const long PAT_OMP_FORK                            = 39;
const long PAT_OMP_TARGET_TIME                     = 40;
const long PAT_OMP_TARGET_TRANSFER                 = 41;
const long PAT_OMP_TARGET_KERNEL_LAUNCHES          = 42;
const long PAT_OMP_SYNCHRONIZATION                 = 43;
const long PAT_OMP_BARRIER                         = 44;
const long PAT_OMP_EBARRIER                        = 45;
const long PAT_OMP_EBARRIER_WAIT                   = 46;
const long PAT_OMP_IBARRIER                        = 47;
const long PAT_OMP_IBARRIER_WAIT                   = 48;
const long PAT_OMP_CRITICAL                        = 49;
const long PAT_OMP_LOCK_CONTENTION_CRITICAL        = 50;
const long PAT_OMP_LOCK_API                        = 51;
const long PAT_OMP_LOCK_CONTENTION_API             = 52;
const long PAT_OMP_ORDERED                         = 53;
const long PAT_OMP_TASKWAIT                        = 54;
const long PAT_OMP_FLUSH                           = 55;
const long PAT_PTHREAD_TIME                        = 56;
const long PAT_PTHREAD_MANAGEMENT                  = 57;
const long PAT_PTHREAD_SYNCHRONIZATION             = 58;
const long PAT_PTHREAD_LOCK_API                    = 59;
const long PAT_PTHREAD_LOCK_CONTENTION_MUTEX_LOCK  = 60;
const long PAT_PTHREAD_CONDITIONAL                 = 61;
const long PAT_PTHREAD_LOCK_CONTENTION_CONDITIONAL = 62;
const long PAT_THREAD_LOCK_CONTENTION              = 63;
const long PAT_OPENACC_TIME                        = 64;
const long PAT_OPENACC_SETUP                       = 65;
const long PAT_OPENACC_COMM                        = 66;
const long PAT_OPENACC_SYNC                        = 67;
const long PAT_OPENACC_KERNEL_LAUNCHES             = 68;
const long PAT_OPENCL_KERNEL_EXECUTIONS            = 69;
const long PAT_OPENCL_TIME                         = 70;
const long PAT_OPENCL_SETUP                        = 71;
const long PAT_OPENCL_COMM                         = 72;
const long PAT_OPENCL_SYNC                         = 73;
const long PAT_OPENCL_KERNEL_LAUNCHES              = 74;
const long PAT_CUDA_KERNEL_EXECUTIONS              = 75;
const long PAT_CUDA_TIME                           = 76;
const long PAT_CUDA_SETUP                          = 77;
const long PAT_CUDA_COMM                           = 78;
const long PAT_CUDA_SYNC                           = 79;
const long PAT_CUDA_KERNEL_LAUNCHES                = 80;
const long PAT_HIP_KERNEL_EXECUTIONS               = 81;
const long PAT_HIP_TIME                            = 82;
const long PAT_HIP_STREAM                          = 83;
const long PAT_HIP_MALLOC                          = 84;
const long PAT_HIP_MEMCPY                          = 85;
const long PAT_HIP_SYNC                            = 86;
const long PAT_HIP_KERNEL_LAUNCHES                 = 87;
const long PAT_SYNCS                               = 88;
const long PAT_SYNCS_P2P                           = 89;
const long PAT_SYNCS_SEND                          = 90;
const long PAT_SYNCS_RECV                          = 91;
const long PAT_SYNCS_COLL                          = 92;
const long PAT_SYNCS_RMA                           = 93;
const long PAT_SYNCS_RMA_ACTIVE                    = 94;
const long PAT_SYNCS_RMA_PASSIVE                   = 95;
const long PAT_COMMS                               = 96;
const long PAT_COMMS_P2P                           = 97;
const long PAT_COMMS_SEND                          = 98;
const long PAT_COMMS_RECV                          = 99;
const long PAT_COMMS_COLL                          = 100;
const long PAT_COMMS_CXCH                          = 101;
const long PAT_COMMS_CSRC                          = 102;
const long PAT_COMMS_CDST                          = 103;
const long PAT_COMMS_RMA                           = 104;
const long PAT_COMMS_RMA_PUTS                      = 105;
const long PAT_COMMS_RMA_GETS                      = 106;
const long PAT_COMMS_RMA_ATOMICS                   = 107;
const long PAT_BYTES                               = 108;
const long PAT_BYTES_P2P                           = 109;
const long PAT_BYTES_SENT                          = 110;
const long PAT_BYTES_RCVD                          = 111;
const long PAT_BYTES_COLL                          = 112;
const long PAT_BYTES_COUT                          = 113;
const long PAT_BYTES_CIN                           = 114;
const long PAT_BYTES_RMA                           = 115;
const long PAT_BYTES_PUT                           = 116;
const long PAT_BYTES_GET                           = 117;
const long PAT_MPI_SLR_COUNT                       = 118;
const long PAT_MPI_SLS_COUNT                       = 119;
const long PAT_MPI_SLSWO_COUNT                     = 120;
const long PAT_MPI_CLR_COUNT                       = 121;
const long PAT_MPI_CLS_COUNT                       = 122;
const long PAT_MPI_CLSWO_COUNT                     = 123;
const long PAT_MPI_FILE_OPS                        = 124;
const long PAT_MPI_FILE_IOPS                       = 125;
const long PAT_MPI_FILE_IROPS                      = 126;
const long PAT_MPI_FILE_IWOPS                      = 127;
const long PAT_MPI_FILE_COPS                       = 128;
const long PAT_MPI_FILE_CROPS                      = 129;
const long PAT_MPI_FILE_CWOPS                      = 130;
const long PAT_MPI_RMA_WAIT_AT_CREATE              = 131;
const long PAT_MPI_RMA_WAIT_AT_FREE                = 132;
const long PAT_MPI_RMA_SYNCHRONIZATION             = 133;
const long PAT_MPI_RMA_SYNC_ACTIVE                 = 134;
const long PAT_MPI_RMA_SYNC_LATE_POST              = 135;
const long PAT_MPI_RMA_EARLY_WAIT                  = 136;
const long PAT_MPI_RMA_LATE_COMPLETE               = 137;
const long PAT_MPI_RMA_WAIT_AT_FENCE               = 138;
const long PAT_MPI_RMA_EARLY_FENCE                 = 139;
const long PAT_MPI_RMA_SYNC_PASSIVE                = 140;
const long PAT_MPI_RMA_SYNC_LOCK_COMPETITION       = 141;
const long PAT_MPI_RMA_SYNC_WAIT_FOR_PROGRESS      = 142;
const long PAT_MPI_RMA_COMMUNICATION               = 143;
const long PAT_MPI_RMA_COMM_LATE_POST              = 144;
const long PAT_MPI_RMA_COMM_LOCK_COMPETITION       = 145;
const long PAT_MPI_RMA_COMM_WAIT_FOR_PROGRESS      = 146;
const long PAT_MPI_RMA_PAIRSYNC_COUNT              = 147;
const long PAT_MPI_RMA_PAIRSYNC_UNNEEDED_COUNT     = 148;
const long PAT_IMBALANCE                           = 149;
const long PAT_IMBALANCE_ABOVE                     = 150;
const long PAT_IMBALANCE_ABOVE_SINGLE              = 151;
const long PAT_IMBALANCE_BELOW                     = 152;
const long PAT_IMBALANCE_BELOW_BYPASS              = 153;
const long PAT_IMBALANCE_BELOW_SINGULARITY         = 154;
const long PAT_STATISTICS                          = 155;
const long PAT_CRITICAL_PATH                       = 156;
const long PAT_CRITICAL_PATH_IMBALANCE             = 157;
const long PAT_PERFORMANCE_IMPACT                  = 158;
const long PAT_PERFORMANCE_IMPACT_CRITICALPATH     = 159;
const long PAT_CRITICAL_PATH_ACTIVITIES            = 160;
const long PAT_CRITICAL_IMBALANCE_IMPACT           = 161;
const long PAT_INTRA_PARTITION_IMBALANCE           = 162;
const long PAT_INTER_PARTITION_IMBALANCE           = 163;
const long PAT_NON_CRITICAL_PATH_ACTIVITIES        = 164;
const long PAT_DELAY                               = 165;
const long PAT_DELAY_MPI                           = 166;
const long PAT_DELAY_P2P                           = 167;
const long PAT_DELAY_LATESENDER_AGGREGATE          = 168;
const long PAT_DELAY_LATESENDER                    = 169;
const long PAT_DELAY_LATESENDER_LONGTERM           = 170;
const long PAT_DELAY_LATERECEIVER_AGGREGATE        = 171;
const long PAT_DELAY_LATERECEIVER                  = 172;
const long PAT_DELAY_LATERECEIVER_LONGTERM         = 173;
const long PAT_DELAY_COLLECTIVE                    = 174;
const long PAT_DELAY_BARRIER_AGGREGATE             = 175;
const long PAT_DELAY_BARRIER                       = 176;
const long PAT_DELAY_BARRIER_LONGTERM              = 177;
const long PAT_DELAY_N2N_AGGREGATE                 = 178;
const long PAT_DELAY_N2N                           = 179;
const long PAT_DELAY_N2N_LONGTERM                  = 180;
const long PAT_DELAY_12N_AGGREGATE                 = 181;
const long PAT_DELAY_12N                           = 182;
const long PAT_DELAY_12N_LONGTERM                  = 183;
const long PAT_DELAY_N21_AGGREGATE                 = 184;
const long PAT_DELAY_N21                           = 185;
const long PAT_DELAY_N21_LONGTERM                  = 186;
const long PAT_DELAY_OMP                           = 187;
const long PAT_DELAY_OMPBARRIER_AGGREGATE          = 188;
const long PAT_DELAY_OMPBARRIER                    = 189;
const long PAT_DELAY_OMPBARRIER_LONGTERM           = 190;
const long PAT_DELAY_OMPIDLE_AGGREGATE             = 191;
const long PAT_DELAY_OMPIDLE                       = 192;
const long PAT_DELAY_OMPIDLE_LONGTERM              = 193;
const long PAT_WAITSTATES_DIRECT_VS_INDIRECT       = 194;
const long PAT_MPI_WAIT_DIRECT                     = 195;
const long PAT_MPI_WAIT_DIRECT_LATESENDER          = 196;
const long PAT_MPI_WAIT_DIRECT_LATERECEIVER        = 197;
const long PAT_MPI_WAIT_INDIRECT                   = 198;
const long PAT_MPI_WAIT_INDIRECT_LATESENDER        = 199;
const long PAT_MPI_WAIT_INDIRECT_LATERECEIVER      = 200;
const long PAT_WAITSTATES_PROPAGATING_VS_TERMINAL  = 201;
const long PAT_MPI_WAIT_PROPAGATING                = 202;
const long PAT_MPI_WAIT_PROPAGATING_LS             = 203;
const long PAT_MPI_WAIT_PROPAGATING_LR             = 204;
const long PAT_MPI_WAIT_TERMINAL                   = 205;
const long PAT_MPI_WAIT_TERMINAL_LS                = 206;
const long PAT_MPI_WAIT_TERMINAL_LR                = 207;


// --- Forward declarations -------------------------------------------------

class AnalyzeTask;


// --- Pattern registration -------------------------------------------------

void
create_patterns(AnalyzeTask* analyzer);
}    // namespace scout


#endif    // !SCOUT_PATTERNS_GEN_H
